/* Lesson 1.8 Demo  - Guideline Samples for Selecting Column Types */

/*
>>>>>>>>>>>>>>>Using the IDENTITY Property>>>>>>>>>>>>>>>>>>>>>>>>>>>
IDENTITY [ (seed ,increment) ] 
seed - value used for very first row loaded into the table
increment - value that is added to identity value of the previous row
*/

CREATE TABLE Production.Categories(
	CategoryID INT IDENTITY(1,1) NOT NULL,
	CategoryName NVARCHAR (40),
	Description NVARCHAR(max)
	CONSTRAINT PK_Categories_CategoryID PRIMARY KEY(CategoryID),
);
GO

/*
>>>>>>>>>>>>>>>>>>Using SEQUENCE>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
A Sequence is preferred over an Identity property when:
A number needed before data inserted into the table
Single series of numbers needs to be shared between multiple tables or multiple columns within a table.
Series needs to be recycled when a specified number is reached
*/
CREATE SEQUENCE CountBy5
   AS tinyint
    START WITH 1
    INCREMENT BY 1
    MINVALUE 1
    MAXVALUE 5
    CYCLE ;
GO

SELECT NEXT VALUE FOR CountBy5 AS SurveyGroup, Name FROM sys.objects ;
GO

/*
>>>>>>>>>>>>>>>>>>>Creating a SEQUENCE used in a table>>>>>>>>>>
*/
USE AdventureWorks2012
GO

--Create the Cert schema
CREATE SCHEMA Cert ;
GO

-- Create a table
CREATE TABLE Cert.Orders
    (OrderID int PRIMARY KEY,
    Name varchar(20) NOT NULL,
    Qty int NOT NULL);
GO

-- Create a sequence
CREATE SEQUENCE Cert.CountBy1
    START WITH 1
    INCREMENT BY 1 ;
GO

-- Insert three records
INSERT Cert.Orders (OrderID, Name, Qty)
    VALUES (NEXT VALUE FOR Cert.CountBy1, 'pineapple', 2) ;
INSERT Cert.Orders (OrderID, Name, Qty)
    VALUES (NEXT VALUE FOR Cert.CountBy1, 'banana', 6) ;
INSERT Cert.Orders (OrderID, Name, Qty)
    VALUES (NEXT VALUE FOR Cert.CountBy1, 'orange', 12) ;
GO

-- View the table
SELECT * FROM Cert.Orders ;
GO

--CLEANUP--
--Drop table Orders and drop schema Cert --
DROP TABLE Cert.Orders;
GO
DROP SEQUENCE Cert.CountBy1
GO
DROP SCHEMA Cert
GO
