/* Lesson 6.4 Demo  - Use Subqueries and Table Expressions in Queries */

-->>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
------------Using CTE--------------------------------------------------
USE AdventureWorks2012;
GO
-- Define the CTE expression name and column list.
WITH Sales_CTE (SalesPersonID, SalesOrderID, SalesYear)
AS
-- Define the CTE query.
(
    SELECT SalesPersonID, SalesOrderID, YEAR(OrderDate) AS SalesYear
    FROM Sales.SalesOrderHeader
    WHERE SalesPersonID IS NOT NULL
)
-- Define the outer query referencing the CTE name.
SELECT SalesPersonID, COUNT(SalesOrderID) AS TotalSales, SalesYear
FROM Sales_CTE
GROUP BY SalesYear, SalesPersonID
ORDER BY SalesPersonID, SalesYear;
GO

-->>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
------------Using Derived Table----------------------------------------
USE AdventureWorks2012
GO

--Derived table - derived_year - is used to retrieve info regarding orders
--placed by distinct customers per year
--Outer query summarizes the results of the orders placed into derived table (derived_year)

SELECT order_year, COUNT(DISTINCT CustomerID) AS customer_count
FROM (SELECT YEAR(OrderDate) AS order_year,CustomerID
	  FROM Sales.SalesOrderHeader) AS derived_year
GROUP BY order_year;
GO

-->>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
--USING CROSS APPLY----------------------------------------------------

USE AdventureWorks2012
GO

--FIND THE LAST THREE ORDERS FOR CUSTOMERS IN TERRITORYID 2
SELECT 
	cust.customerID, cust.AccountNumber,
	ca.*
FROM Sales.Customer AS cust
--USE CROSS APPLY WHICH IS SIMILAR TO LEFT OUTER JOIN BETWEEN TWO TABLES --
--APPLIES THE Sales from the SalesOrderHeader (right) table to each row in the Sales.Customer (left) table --
--CROSS APPLY does not return rows for those with NULL in columns of the right (SalesOrderHeader) table --
CROSS APPLY(
	SELECT TOP(3) soh.Orderdate, soh.SalesOrderID FROM Sales.SalesOrderHeader AS soh
	WHERE soh.CustomerID = cust.CustomerID
	ORDER BY soh.OrderDate DESC
) AS ca
WHERE cust.TerritoryID = 2
GO

-->>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
--USING OUTER APPLY----------------------------------------------------
USE AdventureWorks2012
GO

--FIND THE LAST THREE ORDERS FOR CUSTOMERS IN TERRITORYID 2
SELECT 
	cust.customerID, cust.AccountNumber,
	oa.*
FROM Sales.Customer AS cust
--USE OUTER APPLY WHICH IS SIMILAR TO LEFT OUTER JOIN BETWEEN TWO TABLES --
--APPLIES THE Sales from the SalesOrderHeader (right) table to each row in the Sales.Customer (left) table --
--OUTER APPLY adds rows for those with NULL in columns of the right (SalesOrderHeader) table --
OUTER APPLY(
	SELECT TOP(3) soh.Orderdate, soh.SalesOrderID FROM Sales.SalesOrderHeader AS soh
	WHERE soh.CustomerID = cust.CustomerID
	ORDER BY soh.OrderDate DESC
) AS oa
WHERE cust.TerritoryID = 2
GO